/*
    Парус 8 - Панели мониторинга - УДП - Доски задач
    Компонент: Вкладка информации об исполнителе
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useContext, useCallback } from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import { Box, TextField } from "@mui/material"; //Интерфейсные компоненты
import { ApplicationСtx } from "../../../context/application"; //Контекст приложения
import { getInputProps } from "./task_form"; //Формирование кнопки доступа к разделу
import dayjs from "dayjs"; //Работа с датами
import customParseFormat from "dayjs/plugin/customParseFormat"; //Настройка пользовательского формата даты

//---------
//Константы
//---------

//Стили
const STYLES = {
    BOX_WITH_LEGEND: { border: "1px solid #939393" },
    BOX_SINGLE_COLUMN: { display: "flex", flexDirection: "column", gap: "10px" },
    BOX_LEFT_ALIGN: { display: "flex", justifyContent: "flex-start" },
    LEGEND: { textAlign: "left" }
};

//------------------------------------
//Вспомогательные функции и компоненты
//------------------------------------

//Подключение настройки пользовательского формата даты
dayjs.extend(customParseFormat);

//-----------
//Тело модуля
//-----------

//Вкладка информации об исполнителе
const TaskFormTabExecutor = ({ task, onFieldEdit }) => {
    //Подключение к контексту приложения
    const { pOnlineShowDictionary } = useContext(ApplicationСtx);

    //Отображение раздела "Сотрудники" для инициатора
    const handleClientPersonOpen = useCallback(async () => {
        pOnlineShowDictionary({
            unitCode: "ClientPersons",
            showMethod: "main",
            inputParameters: [{ name: "in_CODE", value: task.sinit_clnperson }],
            callBack: res => {
                res.success
                    ? onFieldEdit({
                          target: {
                              id: "sinit_clnperson",
                              value: res.outParameters.out_CODE
                          }
                      })
                    : null;
            }
        });
    }, [onFieldEdit, pOnlineShowDictionary, task.sinit_clnperson]);

    //Генерация содержимого
    return (
        <Box>
            <Box sx={{ ...STYLES.BOX_WITH_LEGEND, ...STYLES.BOX_LEFT_ALIGN }} component="fieldset">
                <legend style={STYLES.LEGEND}>Планирование</legend>
                <TextField
                    id="dplan_date"
                    label="Начало работ"
                    InputLabelProps={{ shrink: true }}
                    type="datetime-local"
                    variant="standard"
                    value={task.dplan_date ? dayjs(task.dplan_date, "DD.MM.YYYY HH:mm").format("YYYY-MM-DD HH:mm") : ""}
                    onChange={onFieldEdit}
                    disabled={task.isUpdate}
                ></TextField>
            </Box>
            <Box sx={{ ...STYLES.BOX_WITH_LEGEND, ...STYLES.BOX_SINGLE_COLUMN }} component="fieldset">
                <legend style={STYLES.LEGEND}>Инициатор</legend>
                <TextField
                    id="sinit_clnperson"
                    label="Сотрудник"
                    value={task.sinit_clnperson}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled={task.isUpdate}
                    InputProps={getInputProps(() => handleClientPersonOpen(), task.isUpdate)}
                ></TextField>
                <TextField
                    id="sinit_user"
                    label="Пользователь"
                    value={task.sinit_user}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled
                ></TextField>
                <TextField
                    id="sinit_reason"
                    label="Основание"
                    value={task.sinit_reason}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled={task.isUpdate}
                ></TextField>
            </Box>
            <Box sx={{ ...STYLES.BOX_WITH_LEGEND, ...STYLES.BOX_SINGLE_COLUMN }} component="fieldset">
                <legend style={STYLES.LEGEND}>Направить</legend>
                <TextField
                    id="sto_company"
                    label="Организация"
                    value={task.sto_company}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled
                ></TextField>
                <TextField
                    id="sto_department"
                    label="Подразделение"
                    value={task.sto_department}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled
                ></TextField>
                <TextField id="sto_clnpost" label="Должность" value={task.sto_clnpost} variant="standard" onChange={onFieldEdit} disabled></TextField>
                <TextField
                    id="sto_clnpsdep"
                    label="Штатная должность"
                    value={task.sto_clnpsdep}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled
                ></TextField>
                <TextField
                    id="sto_clnperson"
                    label="Сотрудник"
                    value={task.sto_clnperson}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled
                ></TextField>
                <TextField
                    id="sto_fcstaffgrp"
                    label="Нештатная должность"
                    value={task.sto_fcstaffgrp}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled
                ></TextField>
                <TextField id="sto_user" label="Пользователь" value={task.sto_user} variant="standard" onChange={onFieldEdit} disabled></TextField>
                <TextField
                    id="sto_usergrp"
                    label="Группа пользователей"
                    value={task.sto_usergrp}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled
                ></TextField>
            </Box>
        </Box>
    );
};

//Контроль свойств - Вкладка информации об исполнителе
TaskFormTabExecutor.propTypes = {
    task: PropTypes.object.isRequired,
    onFieldEdit: PropTypes.func.isRequired
};

//----------------
//Интерфейс модуля
//----------------

export { TaskFormTabExecutor };
