/*
    Парус 8 - Панели мониторинга - УДП - Доски задач
    Компонент: Вкладка основной информации
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useContext, useCallback } from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import { Box, TextField } from "@mui/material"; //Интерфейсные компоненты
import { ApplicationСtx } from "../../../context/application"; //Контекст приложения
import { getInputProps } from "./task_form"; //Формирование кнопки доступа к разделу

//---------
//Константы
//---------

//Стили
const STYLES = {
    BOX_WITH_LEGEND: { border: "1px solid #939393" },
    BOX_SINGLE_COLUMN: { display: "flex", flexDirection: "column", gap: "10px" },
    BOX_FEW_COLUMNS: { display: "flex", flexWrap: "wrap", justifyContent: "space-between" },
    LEGEND: { textAlign: "left" },
    TEXT_FIELD: (widthVal, greyDisabled = false) => ({
        margin: "4px",
        ...(widthVal ? { width: widthVal } : {}),
        ...(greyDisabled
            ? {
                  "& .MuiInputBase-input.Mui-disabled": {
                      WebkitTextFillColor: "rgba(0, 0, 0, 0.87)"
                  },
                  "& .MuiInputLabel-root.Mui-disabled": {
                      WebkitTextFillColor: "rgba(0, 0, 0, 0.6)"
                  }
              }
            : {})
    })
};

//-----------
//Тело модуля
//-----------

//Вкладка основной информации
const TaskFormTabInfo = ({ task, editable, onFieldEdit, onEventNextNumbGet }) => {
    //Подключение к контексту приложения
    const { pOnlineShowDictionary } = useContext(ApplicationСtx);

    //Отображение раздела "Каталоги" для событий
    const handleCrnOpen = useCallback(async () => {
        pOnlineShowDictionary({
            unitCode: "CatalogTree",
            showMethod: "main",
            inputParameters: [
                { name: "in_DOCNAME", value: "ClientEvents" },
                { name: "in_NAME", value: task.scrn }
            ],
            callBack: res => {
                res.success
                    ? onFieldEdit({
                          target: {
                              id: "scrn",
                              value: res.outParameters.out_NAME
                          }
                      })
                    : null;
            }
        });
    }, [onFieldEdit, pOnlineShowDictionary, task.scrn]);

    //Отображение раздела "Сотрудники" для клиента
    const handleClientPersonOpen = useCallback(async () => {
        pOnlineShowDictionary({
            unitCode: "ClientPersons",
            showMethod: "main",
            inputParameters: [{ name: "in_CODE", value: task.sclnt_clnperson }],
            callBack: res => {
                res.success
                    ? onFieldEdit({
                          target: {
                              id: "sclnt_clnperson",
                              value: res.outParameters.out_CODE
                          }
                      })
                    : null;
            }
        });
    }, [onFieldEdit, pOnlineShowDictionary, task.sclnt_clnperson]);

    //Отображение раздела "Клиенты"
    const handleClientClientsOpen = useCallback(async () => {
        pOnlineShowDictionary({
            unitCode: "ClientClients",
            showMethod: "main",
            inputParameters: [{ name: "in_CLIENT_CODE", value: task.sclnt_clnclients }],
            callBack: res => {
                res.success
                    ? onFieldEdit({
                          target: {
                              id: "sclnt_clnclients",
                              value: res.outParameters.out_CLIENT_CODE
                          }
                      })
                    : null;
            }
        });
    }, [onFieldEdit, pOnlineShowDictionary, task.sclnt_clnclients]);

    //Генерация содержимого
    return (
        <Box>
            <Box sx={STYLES.BOX_WITH_LEGEND} component="fieldset">
                <legend style={STYLES.LEGEND}>Событие</legend>
                <Box sx={STYLES.BOX_FEW_COLUMNS}>
                    <TextField
                        sx={STYLES.TEXT_FIELD()}
                        id="scrn"
                        label="Каталог"
                        fullWidth
                        value={task.scrn}
                        variant="standard"
                        onChange={onFieldEdit}
                        InputProps={getInputProps(handleCrnOpen)}
                        required
                        disabled={task.isUpdate}
                    />
                    <TextField
                        sx={STYLES.TEXT_FIELD("225px")}
                        id="sprefix"
                        label="Префикс"
                        value={task.sprefix}
                        variant="standard"
                        onChange={onFieldEdit}
                        required
                        disabled={task.isUpdate}
                    ></TextField>
                    <TextField
                        sx={STYLES.TEXT_FIELD("225px")}
                        id="snumber"
                        label="Номер"
                        value={task.snumber}
                        variant="standard"
                        onChange={onFieldEdit}
                        required
                        disabled={task.isUpdate}
                        InputProps={getInputProps(onEventNextNumbGet, !task.sprefix || task.isUpdate, "refresh")}
                    ></TextField>
                    <TextField
                        sx={STYLES.TEXT_FIELD("225px", !task.isUpdate)}
                        id="stype"
                        label="Тип"
                        value={task.stype}
                        variant="standard"
                        onChange={onFieldEdit}
                        disabled
                        required
                    ></TextField>
                    <TextField
                        sx={STYLES.TEXT_FIELD("225px", !task.isUpdate)}
                        id="sstatus"
                        label="Статус"
                        value={task.sstatus}
                        variant="standard"
                        disabled
                        required
                        onChange={onFieldEdit}
                    ></TextField>
                    <TextField
                        sx={STYLES.TEXT_FIELD()}
                        fullWidth
                        id="sdescription"
                        label="Описание"
                        value={task.sdescription}
                        variant="standard"
                        onChange={onFieldEdit}
                        disabled={!task.stype || !editable}
                        required
                        multiline
                        minRows={7}
                        maxRows={7}
                    ></TextField>
                </Box>
            </Box>
            <Box sx={{ ...STYLES.BOX_WITH_LEGEND, ...STYLES.BOX_SINGLE_COLUMN }} component="fieldset">
                <legend style={STYLES.LEGEND}>Клиент</legend>
                <TextField
                    sx={STYLES.TEXT_FIELD()}
                    id="sclnt_clnclients"
                    label="Организация"
                    value={task.sclnt_clnclients}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled={!task.stype}
                    InputProps={getInputProps(handleClientClientsOpen, !task.stype)}
                ></TextField>
                <TextField
                    sx={STYLES.TEXT_FIELD()}
                    id="sclnt_clnperson"
                    label="Сотрудник"
                    value={task.sclnt_clnperson}
                    variant="standard"
                    onChange={onFieldEdit}
                    disabled={!task.stype}
                    InputProps={getInputProps(() => handleClientPersonOpen(), !task.stype)}
                ></TextField>
            </Box>
        </Box>
    );
};

//Контроль свойств - Вкладка основной информации
TaskFormTabInfo.propTypes = {
    task: PropTypes.object.isRequired,
    editable: PropTypes.bool.isRequired,
    onFieldEdit: PropTypes.func.isRequired,
    onEventNextNumbGet: PropTypes.func.isRequired
};

//----------------
//Интерфейс модуля
//----------------

export { TaskFormTabInfo };
