/*
    Парус 8 - Панели мониторинга - УДП - Доски задач
    Компонент: Вкладка информации со свойствами
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useContext } from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import { Box, TextField } from "@mui/material"; //Интерфейсные компоненты
import { getInputProps } from "./task_form"; //Формирование кнопки доступа к разделу
import { ApplicationСtx } from "../../../context/application"; //Контекст приложения
import dayjs from "dayjs"; //Работа с датами
import customParseFormat from "dayjs/plugin/customParseFormat"; //Настройка пользовательского формата даты
import { DP_DEFAULT_VALUE, DP_TYPE_PREFIX, DP_IN_VALUE, DP_RETURN_VALUE, validationError, timeFromSqlFormat } from "../layouts"; //Дополнительная разметка и вёрстка клиентских элементов

//---------
//Константы
//---------

//Стили
const STYLES = {
    BOX_WITH_LEGEND: { border: "1px solid #939393" },
    LEGEND: { textAlign: "left" },
    TEXT_FIELD: (widthVal, greyDisabled = false) => ({
        margin: "4px",
        ...(widthVal ? { width: widthVal } : {}),
        ...(greyDisabled
            ? {
                  "& .MuiInputBase-input.Mui-disabled": {
                      WebkitTextFillColor: "rgba(0, 0, 0, 0.87)"
                  },
                  "& .MuiInputLabel-root.Mui-disabled": {
                      WebkitTextFillColor: "rgba(0, 0, 0, 0.6)"
                  }
              }
            : {})
    })
};

//------------------------------------
//Вспомогательные функции и компоненты
//------------------------------------

//Подключение настройки пользовательского формата даты
dayjs.extend(customParseFormat);

//-----------
//Тело модуля
//-----------

//Вкладка информации со свойствами
const TaskFormTabProps = ({ task, docProps, onPropEdit }) => {
    //Подключение к контексту приложения
    const { pOnlineShowDictionary } = useContext(ApplicationСtx);

    //Выбор из словаря или дополнительного словаря
    const handleDict = async (dp, curValue = null) => {
        dp.entryType === 1
            ? pOnlineShowDictionary({
                  unitCode: dp.unitcode,
                  showMethod: dp.showMethodCode,
                  inputParameters: dp.paramRn ? [{ name: dp.paramIn, value: curValue }] : null,
                  callBack: res => {
                      res.success
                          ? onPropEdit({
                                target: {
                                    id: `${DP_TYPE_PREFIX[dp.format]}DP_${dp.rn}`,
                                    value: res.outParameters[dp.paramOut]
                                }
                            })
                          : null;
                  }
              })
            : pOnlineShowDictionary({
                  unitCode: "ExtraDictionaries",
                  showMethod: "values",
                  inputParameters: [
                      { name: "pos_rn", value: dp.extraDictRn },
                      { name: DP_IN_VALUE[dp.format], value: curValue }
                  ],
                  callBack: res => {
                      res.success
                          ? onPropEdit({
                                target: {
                                    id: `${DP_TYPE_PREFIX[dp.format]}DP_${dp.rn}`,
                                    value: res.outParameters[DP_RETURN_VALUE[dp.format]]
                                }
                            })
                          : null;
                  }
              });
    };

    //Инициализация дополнительного свойства
    const initProp = prop => {
        //Значение свойства
        const value = task.docProps[`${DP_TYPE_PREFIX[prop.format]}DP_${prop.rn}`];
        if (
            (task.nrn || task.docProps[`${DP_TYPE_PREFIX[prop.format]}DP_${prop.rn}`]) &&
            task.docProps[`${DP_TYPE_PREFIX[prop.format]}DP_${prop.rn}`] !== undefined
        ) {
            //Строка или число
            if (prop.format < 2) return prop.numPrecision ? String(value).replace(".", ",") : value;
            //Дата
            else if (prop.format === 2) {
                //Дата без времени
                if (prop.dataSubtype === 0) return dayjs(value).format("YYYY-MM-DD");
                //Дата + время без секунд
                else if (prop.dataSubtype === 1) return dayjs(value).format("YYYY-MM-DD HH:mm");
                //Дата + время с секундами
                else return dayjs(value).format("YYYY-MM-DD HH:mm:ss");
            }
            //Время
            else {
                return timeFromSqlFormat(value);
            }
        } else if (task.nrn) {
            return "";
        } else return prop[DP_DEFAULT_VALUE[prop.format]];
    };

    //Генерация содержимого
    return (
        <Box>
            <Box sx={STYLES.BOX_WITH_LEGEND} component="fieldset">
                {docProps.props.map(dp => {
                    return dp.showInGrid ? (
                        <TextField
                            error={
                                !validationError(
                                    task.docProps[`${DP_TYPE_PREFIX[dp.format]}DP_${dp.rn}`],
                                    dp.format,
                                    dp.numWidth,
                                    dp.numPrecision,
                                    dp.strWidth
                                )
                            }
                            key={dp.id}
                            sx={STYLES.TEXT_FIELD()}
                            id={`${DP_TYPE_PREFIX[dp.format]}DP_${dp.rn}`}
                            type={dp.format < 2 ? "string" : dp.format === 2 ? (dp.dataSubtype === 0 ? "date" : "datetime-local") : "time"}
                            label={dp.name}
                            fullWidth
                            value={initProp(dp)}
                            variant="standard"
                            onChange={onPropEdit}
                            inputProps={(dp.format === 2 && dp.dataSubtype === 2) || (dp.format === 3 && dp.dataSubtype === 1) ? { step: 1 } : {}}
                            InputProps={
                                dp.entryType > 0
                                    ? getInputProps(() => handleDict(dp, task.docProps[`${DP_TYPE_PREFIX[dp.format]}DP_${dp.rn}`]))
                                    : null
                            }
                            InputLabelProps={
                                dp.format < 2
                                    ? {}
                                    : {
                                          shrink: true
                                      }
                            }
                            required={dp.require}
                            disabled={dp.readonly}
                        />
                    ) : null;
                })}
            </Box>
        </Box>
    );
};

//Контроль свойств - Вкладка информации со свойствами
TaskFormTabProps.propTypes = {
    task: PropTypes.object.isRequired,
    docProps: PropTypes.object.isRequired,
    onPropEdit: PropTypes.func.isRequired
};

//----------------
//Интерфейс модуля
//----------------

export { TaskFormTabProps };
