/*
    Парус 8 - Панели мониторинга - УДП - Доски задач
    Пользовательские хуки: Хуки фильтра
*/

//---------------------
//Подключение библиотек
//---------------------

import { useState, useEffect, useCallback } from "react"; //Классы React
import { EVENT_STATES } from "../layouts"; //Перечисление состояний события

//-----------
//Тело модуля
//-----------

//Хук фильтра
const useFilters = filterOpen => {
    //Состояние фильтра
    const [filters, setFilters] = useState({
        loaded: false,
        isSetByUser: false,
        needSave: false,
        values: {
            evState: EVENT_STATES[1],
            type: "",
            catalog: "",
            crn: "",
            wSubcatalogs: false,
            sendPerson: "",
            sendDivision: "",
            sendUsrGrp: "",
            docLink: ""
        },
        fArray: [
            { name: "NCLOSED", from: 0, to: 1 },
            { name: "SEVTYPE_CODE", from: "", to: "" },
            { name: "NCRN", from: "", to: "" },
            { name: "SSEND_PERSON", from: "", to: "" },
            { name: "SSEND_DIVISION", from: "", to: "" },
            { name: "SSEND_USRGRP", from: "", to: "" },
            { name: "NLINKED_RN", from: "", to: "" }
        ]
    });

    //Изменение фильтра
    const handleFiltersChange = filters => {
        setFilterValues(filters);
    };

    //Установить значение фильтра
    const setFilterValues = (values, ns = true) => {
        //Считываем массив фильтров
        let filtersArr = filters.fArray.slice();
        //Состояние
        if (values.evState) {
            if (values.evState === EVENT_STATES[0]) {
                filtersArr.find(f => f.name === "NCLOSED").from = 0;
                filtersArr.find(f => f.name === "NCLOSED").to = 1;
            } else if (values.evState === EVENT_STATES[1]) {
                filtersArr.find(f => f.name === "NCLOSED").from = 0;
                filtersArr.find(f => f.name === "NCLOSED").to = 0;
            } else if (values.evState === EVENT_STATES[2]) {
                filtersArr.find(f => f.name === "NCLOSED").from = 1;
                filtersArr.find(f => f.name === "NCLOSED").to = 1;
            }
        }
        //Тип
        filtersArr.find(f => f.name === "SEVTYPE_CODE").from = values.type ? values.type : null;
        //Каталог
        filtersArr.find(f => f.name === "NCRN").from = values.crn ? values.crn : null;
        //Исполнитель
        filtersArr.find(f => f.name === "SSEND_PERSON").from = values.sendPerson ? values.sendPerson : null;
        //Подразделение
        filtersArr.find(f => f.name === "SSEND_DIVISION").from = values.sendDivision ? values.sendDivision : null;
        //Группа пользователей
        filtersArr.find(f => f.name === "SSEND_USRGRP").from = values.sendUsrGrp ? values.sendUsrGrp : null;
        //Учётный документ
        filtersArr.find(f => f.name === "NLINKED_RN").from = values.docLink ? values.docLink : null;
        //Устанавливаем фильтры
        setFilters({ loaded: true, isSetByUser: true, needSave: ns, values: values, fArray: filtersArr });
    };

    //Загрузка значений фильтра из локального хранилища браузера
    const loadLocalFilter = useCallback(async () => {
        let vs = { ...filters.values };
        if (localStorage.getItem("type")) {
            Object.keys(vs).map(function (k) {
                if (k === "wSubcatalogs") vs[k] = localStorage.getItem(k) === "true";
                else k !== "docLink" ? (vs[k] = localStorage.getItem(k)) : null;
            });
            setFilterValues(vs, false);
            filterOpen(false);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);

    //При закрытии панели
    useEffect(() => {
        filters.needSave
            ? window.addEventListener("beforeunload", function () {
                  Object.keys(filters.values).map(function (k) {
                      k !== "docLink" ? localStorage.setItem(k, filters.values[k] ? filters.values[k] : "") : null;
                  });
              })
            : null;
    }, [filters.needSave, filters.values]);

    //При отсутствии пользовательских настроек фильтра
    useEffect(() => {
        if (!filters.isSetByUser) filterOpen(true);
    }, [filterOpen, filters.isSetByUser]);

    //При подключении к странице
    useEffect(() => {
        localStorage.length ? loadLocalFilter() : null;
    }, [loadLocalFilter]);

    return [filters, handleFiltersChange];
};

//----------------
//Интерфейс модуля
//----------------

export { useFilters };
